
/*
 *  Integer - Implements an integer of arbitrary size
 *  Copyright (c) 2005-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See integer.h
 */

/*
News
----

v5  2006-01-19 - 2006-01-20  &  2006-02-23  &  2006-04-04
--

	Added the buf variable to make Integer class not to use dynamic memory on small numbers (below 2^32)
	  (ump got ~70% faster when working on thoose numbers)
	Removed assembler code as it no longer did any noticable difference ( as divide no longer uses thoose functions )
	Added special functions: add_uint32_shifted, sub_uint32_mult_shifted, compare_abs_value_shifted
	Rewrote Integer::divide using the added special functions

v4  2005-10-05 - 2005-11-06
--

	Fixed a bug in add_data
	New versioning system
	Made the floating points functions use floatx
	Converted append function to use string instead of stream

v0.1.2  2005-07-22 - 2005-08-08
------
	Did some optimization in Integer::divide & Integer::mul_by_2_pow_32
	Added sub_data_reverse,   this = nr - this
	Wrote parts of Integer::div_by_2 in asm

v0.1.1  2005-06-13 - 2005-06-15  &  2005-07-09
------
	Wrote parts of Integer::add_data & sub_data in asm
	Made the floating point functions depend on USE_FLOAT, USE_DOUBLE & USE_LONG_DOUBLE

v0.1.0  2005-04 - 2005-06-12
------
	Started the work

*/



#include "integer.h"
#include <math.h>
#include <stdio.h>


void Integer::append_to_string( std::string &str, uint32 nrofdigits ) const throw(error_obj)
{
	try
	{
		std::string str1(""), str2(""), *ptr, *ptr2, *tmp_ptr;
		Integer nr, rest, divider, tmp;
		char char_str[25];

		divider = int64(1000000000)*int64(1000000000);

		ptr = &str1;
		ptr2 = &str2;
		if( used == 0 )
			str1 = "0";
		else
		{
			if( used >= 5 )
			{
				uint32 rest_digits;

				rest = int32( float((used-1)/4 +1)*1.07 );
				rest_digits = rest.data[0]*18;

				divider.power( rest, &tmp );
				this->divide( tmp, &nr, &rest );
				rest.negative = false;

				if( nrofdigits < rest_digits )
					nr.append_to_string( str );
				else
					nr.append_to_string( str, nrofdigits - rest_digits );
				rest.append_to_string( str, rest_digits );        
				return;
			}
			else
			{
				nr = *this;

				do
				{
					nr.divide( divider, &tmp, &rest );
					nr.swap( tmp );

					if( nr == 0 )
					{
						if( rest.used == 1 )
							snprintf( char_str, 25, "%lu", rest.data[0] );
						else
							snprintf( char_str, 25, "%llu", uint64(rest.data[0])+(uint64(rest.data[1])<<32) );
					}
					else
					{
						if( rest.used == 0 )
							snprintf( char_str, 25, "000000000000000000" );
						else if( rest.used == 1 )
							snprintf( char_str, 25, "%018llu", uint64(rest.data[0])  );
						else
							snprintf( char_str, 25, "%018llu", uint64(rest.data[0])+(uint64(rest.data[1])<<32) );
					}
					tmp_ptr = ptr;
					ptr = ptr2;
					ptr2 = tmp_ptr;

					*ptr = char_str;
					ptr->append( *ptr2 );
				}while( nr != 0 );

				if( negative )
				{
					tmp_ptr = ptr;
					ptr = ptr2;
					ptr2 = tmp_ptr;

					*ptr = "-";
					ptr->append( *ptr2 );
				}
			}
		}

		if( (*ptr)[0] != '-' )
		{
			if( ptr->length() < nrofdigits ) // the number of digit is not met,
			{                                // lets add some starting zero's
				tmp_ptr = ptr;
				ptr = ptr2;
				ptr2 = tmp_ptr;

				*ptr = "";
				while( ptr2->length() < nrofdigits )
				{
					nrofdigits--;
					ptr->append( "0" );
				}
				ptr->append( *ptr2 );
			}
		}

		str.append( *ptr );
	}
	catch(error_obj error) {  throw error;                 }
	catch(...)             {  Throw_Couldnt_get_memory();  }
}

// gcd - greatest common divisior (using Euklides algorithm)
// if any of nr1 & nr2 is zero the return value is zero also
Integer gcd( Integer nr1, Integer nr2 ) throw(error_obj)
{
	if( nr1.used == 0  ||  nr2.used == 0 )
		return 0;

	if( nr1.negative )
		nr1.negate();
	if( nr2.negative )
		nr2.negate();

	{
		Integer tmp;

		if(nr2 > nr1) // this makes nr1 to be the largest number
			goto gcd_nr2_is_bigger;
		for(;;)
		{
			nr1.divide( nr2, 0, &tmp );
			if(tmp.used == 0)
				return nr2;
			nr1.swap( tmp );

gcd_nr2_is_bigger:
			nr2.divide( nr1, 0, &tmp );
			if(tmp.used == 0)
				return nr1;
			nr2.swap( tmp );
		}
	}
}
// lcm - least common multiple ( (nr1*nr2)/gcd(nr1,nr2) )
// if(gcd(nr1,nr2) == 0) the return value is zero also
Integer lcm( const Integer &nr1, const Integer &nr2 ) throw(error_obj)
{
	Integer tmp;

	tmp = gcd(nr1,nr2);
	if(tmp == 0)
		return 0;

	return (nr1 / tmp) * nr2;
}

Integer::Integer(floatx src) throw(error_obj)
{
	uint32 i;
	floatx tmp,tmp2;

	negative = false;
	size = 0;
	used = 0;
	data = 0;

	if( src <= -1.0 )
		size = uint32( (log2x( -src ) + 1.0) / 32.0 ) + 1;
	else if( src < 1.0  &&  src > -1.0 )
	{
		size = 0;
		return;
	}
	else
		size = uint32( (log2x( src ) + 1.0) / 32.0 ) + 1;

	if( size > 2 )
	{
		try{  data = new uint32[size];  }
		catch(...) {  Throw_Couldnt_get_memory();  }
	}
	else
		data = buf;

	used = size;
	if( src < 0.0 )
	{
		negative = true;
		src = -src;
	}
	src = truncx( src );

	for( i=1; i<=size; i++ )
	{
		if( size == i )
			tmp2 = truncx(src);
		else
		{
			tmp = powx( (floatx(uint32_max)+1.0), floatx(size-i) );
	
			tmp2 = truncx(src/tmp);
		}
		data[size-i] = uint32( tmp2 );
	
		src = truncx( src - tmp2*tmp );
	}

	if( data[used-1] == 0 )
		used--;
}
Integer::operator floatx(void)
{
	floatx res;

	switch( used )
	{
	case 0:
		return 0.0;

	case 1:
		res = floatx( *data );
		break;

	default:
		res = 0.0;
		for( uint32 i=0; i < used  &&  i < 3 ; i++ )
			res += floatx(data[used-i-1]) * powx( float(uint32_max)+1.0f, floatx(used-i-1) );
	}

	if( negative )
		return -res;
	 return res;
}
Integer & Integer::operator=(floatx val) throw(error_obj)
{
	uint32 i, new_size;
	floatx tmp,tmp2;

	if( val <= -1.0 )
		new_size = uint32( (log2x( -val ) + 1.0) / 32.0 ) + 1;
	else if( val < 1.0  &&  val > -1.0 )
	{
		used = 0;
		return *this;
	}
	else
		new_size = uint32( (log2x( val ) + 1.0) / 32.0 ) + 1;

	if( new_size > size )
	{
		if( size > 2 )
			delete [] data;
		data = 0;
		used = 0;
		size = 0;

		if( new_size > 2 )
		{
			try{  data = new uint32[new_size];  }
			catch(...) {  Throw_Couldnt_get_memory();  }
		}
		else
			data = buf;

		size = new_size;
	}

	used = new_size;
	if( val < 0.0 )
	{
		negative = true;
		val = -val;
	}
	val = truncx( val );

	for( i=1; i<=new_size; i++ )
	{
		if( new_size == i )
			tmp2 = truncx(val);
		else
		{
			tmp = powx( (floatx(uint32_max)+1.0), floatx(new_size-i) );

			tmp2 = truncx(val/tmp);
		}
		data[new_size-i] = uint32( tmp2 );

		val = truncx( val - tmp2*tmp );
	}

	if( data[used-1] == 0 )
		used--;
	return *this;
}




Integer::Integer()
{
	negative = false;
	size = 0;
	data = 0;
	used = 0;
}

Integer::Integer( const Integer &src ) throw(error_obj)
{
	negative = false;
	size = 0;
	used = 0;
	data = 0;

	this->operator=( src );
}

Integer::Integer(int32 src)
{
	negative = false;
	size = 0;
	used = 0;
	data = 0;

	if( src == 0 )
		return;

	data = buf;

	size = 1;
	used = 1;
	if( src < 0 )
	{
		negative = true;
		data[0] = uint32( -src );
	}
	else
	{
		negative = false;
		data[0] = uint32( src );
	}
}

Integer::Integer(int64 src)
{
	negative = false;
	size = 0;
	used = 0;
	data = 0;

	if( src == 0 )
		return;

	data = buf;

	size = 2;
	used = 2;

	if( src < 0 )
	{
		negative = true;
		data[1] = uint32( uint64(-src) >> 32 );
		data[0] = uint32( uint64(-src) );
	}
	else
	{
		negative = false;
		data[1] = uint32( uint64(src) >> 32 );
		data[0] = uint32( uint64(src) );
	}
	if( data[1] == 0 )
		used = 1;
}

Integer::~Integer()
{
	if( size > 2 )
		delete [] data;
}

Integer::operator int64(void)
{
	int64 res;

	if( used == 0 )
		return int64(0);

	if( used >= 3 )
		res = int64_max;
	else
	{
		res = data[0];

		if( used == 2 )
		{
			if( data[1] >= (uint32(1)<<31) )
				res = int64_max;
			else
				res += int64(data[1])*(int64(1)<<32);
		}
	}

	if( negative )
		return -res;
	return res;  
}

Integer & Integer::operator=(int32 src)
{
	if( src == 0 )
	{
		used = 0;
		return *this;
	}

	enlarge_to( 1 );

	used = 1;
	if( src < 0 )
	{
		negative = true;
		data[0] = uint32( -src );
	}
	else
	{
		negative = false;
		data[0] = uint32( src );
	}
	return *this;
}

Integer & Integer::operator=(int64 src)
{
	if( src == 0 )
	{
		used = 0;
		return *this;
	}

	enlarge_to( 2 );

	used = 2;

	if( src < 0 )
	{
		negative = true;
		data[1] = uint32( uint64(-src) >> 32 );
		data[0] = uint32( uint64(-src) );
	}
	else
	{
		negative = false;
		data[1] = uint32( uint64(src) >> 32 );
		data[0] = uint32( uint64(src) );
	}
	if( data[1] == 0 )
		used = 1;

	return *this;
}

Integer & Integer::operator=(const Integer &val) throw(error_obj)
{
	if( size < val.used )
	{
		if( size > 2 )
			delete [] data;
		size = 0;
		used = 0;
		data = 0;

		if( val.used > 2 )
		{
			try{  data = new uint32[val.used];  }
			catch(...) {  Throw_Couldnt_get_memory();  }
		}
		else
			data = buf;

		size = val.used;
	}
	used = val.used;
	negative = val.negative;

	for( uint32 i=0; i<used; i++ )
		data[i] = val.data[i];

	return *this;
}

bool Integer::operator==(const Integer &val) const
{
	if( used != val.used )
		return false;
	if( used == 0 )
		return true;

	if( negative != val.negative )
		return false;

	for( uint32 i=0; i<used; i++ )
	{
		if( data[i] != val.data[i] )
			return false;
	}
	return true;
}

bool Integer::operator==(int32 val) const
{
	uint32 tmp;

	if( used == 0 )
		return (val == 0);

	if( used > 1 )
		return false;

	if( negative )
	{
		if( val < 0 )
			tmp = uint32(-int64(val));
		else
			return false;
	}
	else
	{
		if( val < 0 )
			return false;
		tmp = uint32(val);
	}

	return ( data[0] == tmp );
}

// -1 this is smaller
//  0 they are equal
//  1 this is bigger
int Integer::compare_abs_value( const Integer &val ) const
{
	if( used == 0 )
	{
		if( val.used == 0 )
			return 0;
		return -1;
	}

	if( used < val.used )
		return -1;
	if( used > val.used )
		return 1;

	for( uint32 i=used; i>0; i-- )
	{
		if( data[i-1] != val.data[i-1] )
			return ( data[i-1] < val.data[i-1] ) ? -1 : 1;
	}
	return 0;
}

int Integer::compare_abs_value_shifted( const Integer &val, uint32 shift ) const
{
	uint32 i;
	if( used == 0 )
	{
		if( (val.used+shift) == 0 )
			return 0;
		return -1;
	}

	if( used < (val.used+shift) )
		return -1;
	if( used > (val.used+shift) )
		return 1;

	for( i=used; i>shift; i-- )
	{
		if( data[i-1] != val.data[i-shift-1] )
			return ( data[i-1] < val.data[i-shift-1] ) ? -1 : 1;
	}
	for( ; i>0; i-- )
	{
		if( data[i-1] != 0 )
			return 1;
	}
	return 0;
}

bool Integer::operator<(const Integer &val) const
{
	if( used == 0 )
	{
		if( val.used == 0 )
			return false;

		return !val.negative;
	}

	if( negative != val.negative )
		return negative;

	if( used < val.used )
		return !negative;
	if( used > val.used )
		return negative;

	for( uint32 i=used; i>0; i-- )
	{
		if( data[i-1] != val.data[i-1] )
		{
			if( negative )
				return ( data[i-1] > val.data[i-1] );
			return ( data[i-1] < val.data[i-1] );
		}
	}

	return false;
}

bool Integer::operator<(int32 val) const
{
	uint32 tmp;

	if( used == 0 )
	{
		if( val == 0 )
			return false;

		return (val > 0);
	}

	if( negative != (val < 0) )
		return negative;

	if( used > 1 )
		return false;

	if( val < 0 )
		tmp = uint32( (-int64(val))+int64(1) );
	else
		tmp = uint32(val);

	if( negative )
		return ( data[0] > tmp );
	return ( data[0] < tmp );
}

bool Integer::operator>(const Integer &val) const
{
	if( used == 0 )
	{
		if( val.used == 0 )
			return false;

		return val.negative;
	}

	if( negative != val.negative )
		return !negative;

	if( used < val.used )
		return negative;
	if( used > val.used )
		return !negative;

	for( uint32 i=used; i>0; i-- )
	{
		if( data[i-1] != val.data[i-1] )
		{
			if( negative )
				return ( data[i-1] < val.data[i-1] );
			return ( data[i-1] > val.data[i-1] );
		}
	}

	return false;
}

bool Integer::operator>(int32 val) const
{
	uint32 tmp;

	if( used == 0 )
	{
		if( val == 0 )
			return false;

		return (val < 0);
	}

	if( negative != (val < 0) )
		return !negative;

	if( used > 1 )
		return true;

	if( val < 0 )
		tmp = uint32( (-int64(val))+int64(1) );
	else
		tmp = uint32(val);

	if( negative )
		return ( data[0] < tmp );
	return ( data[0] > tmp );
}

Integer operator-(const Integer &val) throw(error_obj)
{
	Integer tmp;

	tmp = val;
	tmp.negative = !tmp.negative;

	return tmp;
}

void Integer::enlarge_to(uint32 new_size) throw(error_obj)
{
	if( size < new_size )
	{
		if( new_size > 2 )
		{
			uint32 *new_data;
			bool use_delete = ( size > 2 );

			try{  new_data = new uint32[new_size];  }
			catch(...) {  Throw_Couldnt_get_memory();  }

			size = new_size;

			for( uint32 i=0; i<used; i++ )
				new_data[i] = data[i];

			if( use_delete )
				delete [] data;
			data = new_data;
		}
		else
		{
			size = new_size;
			data = buf;
		}
	}
}

Integer & Integer::operator++(void) throw(error_obj)
{
	if( used == 0 )
	{
		enlarge_to( 1 );
		data[0] = 1;
		used = 1;
		negative = false;
		return *this;
	}

	if( negative )
	{
		for( uint32 i=0; ; i++ )
		{
			data[i]--;
			if( data[i] != uint32_max )
				break;
		}
		if( data[used-1] == 0 )
			used--;
	}
	else
	{
		if( data[used-1] == uint32_max )
			enlarge_to( used+1 );

		for( uint32 i=0; ; i++ )
		{
			if( i == used )
			{
				data[i] = 1;
				used++;
				break;
			}
			data[i]++;
			if( data[i] != 0 )
				break;
		}
	}

	return *this;
}

Integer & Integer::operator--(void) throw(error_obj)
{
	if( used == 0 )
	{
		enlarge_to( 1 );
		data[0] = 1;
		used = 1;
		negative = true;
		return *this;
	}
	else if( data[used-1] == uint32_max )
		enlarge_to( used+1 );

	if( !negative )
	{
		for( uint32 i=0; ; i++ )
		{
			data[i]--;
			if( data[i] != uint32_max )
				break;
		}
		if( data[used-1] == 0 )
			used--;
	}
	else
	{
		for( uint32 i=0; ; i++ )
		{
			if( i == used )
			{
				data[i] = 1;
				used = i+1;
				break;
			}
			data[i]++;
			if( data[i] != 0 )
				break;
		}
	}

	return *this;
}

void Integer::add_data( const Integer &nr ) throw(error_obj)
{
	uint32 i;

	if( nr.used == 0 ) // nothing to add
		return;

	if( nr.used >= size )
	{
		enlarge_to( nr.used + 1 );
		for( i=used; i<(nr.used+1); i++ )
			data[i] = 0;
	}
	else if( size == used )
	{
		enlarge_to( size + 2 );
		data[used] = 0;
		used++;
	}

	if( used <= nr.used )
	{
		for( i=used; i<=nr.used; i++ )
			data[i] = 0;
		used = nr.used + 1;
	}

	{
		uint32 overflow=0, low_used, *ptr, *ptr2;

		ptr = data;
		ptr2 = nr.data;
		low_used = ( used < nr.used ) ? used : nr.used;
		for( i=0; i<low_used; i++, ptr++, ptr2++ )
		{
			if( (*ptr += *ptr2 + overflow) <= *ptr2 )
			{
				if( *ptr < *ptr2 )
					overflow = 1;
				//else // *ptr == *ptr2
				//	overflow = overflow;
			}
			else
				overflow = 0;
		}
		if( overflow )
		{
			i--;
			do
			{
				i++;
				data[i]++;
			}while( data[i] == 0 );
		}
	}

	if( used != 0 )
		if( data[used-1] == 0 )
			used--;
}

void Integer::sub_data_reverse( const Integer &nr )
{
	Integer res;
	res = nr;
	res.sub_data( *this );
	*this = res;
}

//
// this must be larger than nr....no checking for that will be done
//
void Integer::sub_data( const Integer &nr )
{
	uint32 i;

	{
		uint64 tmp, mask, borrowed=0;
		uint32 *ptr, *ptr2;
		mask = uint64( uint32_max );

		ptr = data;
		ptr2 = nr.data;
		for( i=0; i<nr.used; i++, ptr++, ptr2++ )
		{
			tmp = uint64(*ptr) - uint64(*ptr2) - borrowed;
			if( tmp > mask )
			{
				borrowed = 1;
				tmp &= mask;
			}
			else
				borrowed = 0;

			*ptr = uint32(tmp);
		}
		if( borrowed )
		{
			for( ; ; i++ )
			{
				data[i]--;
				if( data[i] != uint32_max )
					break;
			}
		}
	}

	for( i=used-1; i != uint32_max; i-- )
	{
		if( data[i] != 0 )
			break;
	}
	used = i+1;
}

Integer Integer::operator+(const Integer &val) const throw(error_obj)
{
	Integer res;

	if( negative == val.negative )
	{
		res = *this;
		res.add_data( val );
		res.negative = negative;
	}
	else
	{
		if( val.negative )
		{
			if( compare_abs_value( val ) >= 0 )
			{
				res = *this;
				res.sub_data( val );
				res.negative = false;
			}
			else
			{
				res = val;
				res.sub_data( *this );
				res.negative = true;
			}
		}
		else
		{
			if( compare_abs_value( val ) > 0 )
			{
				res = *this;
				res.sub_data( val );
				res.negative = true;
			}
			else
			{
				res = val;
				res.sub_data( *this );
				res.negative = false;
			}
		}
	}
	return res;
}

Integer Integer::operator-(const Integer &val) const throw(error_obj)
{
	Integer res;

	if( negative != val.negative )
	{
		res = *this;
		res.add_data( val );
		res.negative = negative;
	}
	else
	{
		if( !negative )
		{
			if( compare_abs_value( val ) >= 0 )
			{
				res = *this;
				res.sub_data( val );
				res.negative = false;
			}
			else
			{
				res = val;
				res.sub_data( *this );
				res.negative = true;
			}
		}
		else
		{
			if( compare_abs_value( val ) > 0 )
			{
				res = *this;
				res.sub_data( val );
				res.negative = true;
			}
			else
			{
				res = val;
				res.sub_data( *this );
				res.negative = false;
			}
		}
	}
	return res;
}

void Integer::mul_by_2(void) throw(error_obj)
{
	if( used == 0 )
		return;

	if( data[used-1] & (uint32(1)<<31) )
	{
		enlarge_to( used + 1 );
		data[used] = 0;
		used++;
	}

	for( uint32 i=used-1; i>0; i-- )
	{
		data[i] <<= 1;
		if( data[i-1] & (uint32(1)<<31) )
			data[i]++;
	}
	data[0] <<= 1;
}

void Integer::mul_by_2_pow_32(uint32 n) throw(error_obj)
{
	if( used == 0  ||  n == 0 )
		return;

	if( size < (used+n) )
	{
		uint32 *new_data;
		bool use_delete = ( size > 2 );

		try{  new_data = new uint32[used+n+1];  }
		catch(...) {  Throw_Couldnt_get_memory(); }

		size = used+n+1;

		for( uint32 i=used+n-1; i>(n-1); i-- )
			new_data[i] = data[i-n];

		for( uint32 i=0; i<n; i++ )
			new_data[i] = 0;

		used += n;
		if( use_delete )
			delete [] data;
		data = new_data;
	}
	else
	{
		for( uint32 i=used+n-1; i>(n-1); i-- )
			data[i] = data[i-n];

		for( uint32 i=0; i<n; i++ )
			data[i] = 0;
		used += n;
	}
}

void Integer::div_by_2(void) throw(error_obj)
{
	if( used == 0 )
		return;

	data[0] >>= 1;
	for( uint32 i=1; i<used; i++ )
	{
		data[i-1] += ( (data[i] & uint32(1)) << 31);
		data[i] >>= 1;
	}

	if( data[used-1] == 0 )
		used--;
}

void Integer::div_by_2_pow_32(void) throw(error_obj)
{
	if( used == 0 )
		return;

	used--;
	for( uint32 i=0; i<used; i++ )
		data[i] = data[i+1];
}

// special function, used by divide      *this -= nr * 2^(32*shift) * mult
// this must be larger than   nr * 2^(32*shift) * mult
void Integer::sub_uint32_mult_shifted( uint32 mult, const Integer &nr, uint32 shift )
{
	uint64 tmp, mult64, mask, borrowed=0, tmp_hi;
	uint32 i;

	mult64 = uint64(mult);
	mask = uint64( uint32_max );

	for( i=0; i<nr.used; i++ )
	{
		tmp = mult64 * uint64(nr.data[i]);
		if( tmp > mask )
		{
			tmp_hi = tmp >> 32;
			tmp &= mask;
		}
		else
			tmp_hi = 0;

		tmp = uint64(data[i+shift]) - tmp - borrowed;
		if( tmp > mask )
		{
			borrowed = 1;
			tmp &= mask;
		}
		else
			borrowed = 0;

		data[i+shift] = uint32(tmp);

		if( tmp_hi )
		{
			tmp = uint64(data[i+shift+1]) - tmp_hi;
			if( tmp > mask )
			{
				data[i+shift+1] = uint32(tmp & mask);
				for( uint32 i2=i+2; ; i2++ )
				{
					data[i2+shift]--;
					if( data[i2+shift] != uint32_max )
						break;
				}
			}
			else
				data[i+shift+1] = uint32(tmp);
		}
	}
	if( borrowed )
	{
		for( ; ; i++ )
		{
			data[i+shift]--;
			if( data[i+shift] != uint32_max )
				break;
		}
	}
	while( used != 0 )
	{
		if( data[used-1] == 0 )
			used--;
		else
			break;
	}
}

// special function, used by divide      *this += nr * 2^(32*shift)
// this must be large enough and the data should be zero filled
void Integer::add_uint32_shifted( uint32 nr, uint32 shift ) throw(error_obj)
{
	data[shift] += nr;
	if( data[shift] < nr ) // overflowed
	{
		do{
			shift++;
			data[shift]++;
		}while( data[shift] == 0 );
	}
}

void Integer::divide( const Integer &divider, Integer *ans, Integer *rest ) const throw(error_obj)
{
	if( divider.used == 0 )
		Throw_Divide_by_zero();

	if( ans == 0  &&  rest == 0 )
		return;

	if( ans )
		*ans = 0;

	if( this->compare_abs_value( divider ) < 0 )
	{
		if( rest )
			*rest = *this;
		return;
	}

	if( ans )
	{
		ans->enlarge_to( used - divider.used + 1 );

		for( uint32 i=0; i<ans->size; i++ )
			ans->data[i] = 0;
	}

	if( used <= 2  &&  divider.used <= 2 )
	{
		uint64 tmp_nr, tmp_div, res;

		tmp_nr = uint64( data[0] );
		if( used == 2 )
			tmp_nr += uint64( data[1] ) << 32;

		tmp_div = uint64( divider.data[0] );
		if( divider.used == 2 )
			tmp_div += uint64( divider.data[1] ) << 32;

		res = tmp_nr / tmp_div;

		if( ans )
		{
			ans->data[0] = uint32( res & uint64(uint32_max) );
			if( res > uint64(uint32_max) )
			{
				ans->data[1] = uint32( res >> 32 );
				ans->used = 2;
			}
			else
				ans->used = 1;
				ans->negative = negative ^ divider.negative;
		}

		if( rest )
		{
			rest->enlarge_to( 2 );

			tmp_nr -= res*tmp_div;
			rest->data[0] = uint32( tmp_nr & uint64(uint32_max) );
			if( tmp_nr > uint64(uint32_max) )
			{
				rest->data[1] = uint32( tmp_nr >> 32 );
				rest->used = 2;
			}
			else
				rest->used = ( rest->data[0] != 0 );
			rest->negative = negative;
		}
	}
	else
	{
		Integer tnr( *this ), tdiv( divider );
		uint32 appr_mult, tdiv_shift;
		uint64 div;

		if( ans )
		{
			ans->enlarge_to( tnr.used - tdiv.used +1 );
			ans->used = tnr.used - tdiv.used +1;
			for( uint32 i=0; i<ans->used; i++ )
				ans->data[i] = 0;
		}

		div = uint64(tdiv.data[tdiv.used-1]);
		div++;

		tdiv_shift = tnr.used - tdiv.used;
		while( tnr.compare_abs_value( tdiv ) >= 0 )
		{
			while( tnr.compare_abs_value_shifted( tdiv, tdiv_shift ) < 0 )
				tdiv_shift--;

			if( tnr.used > (tdiv.used + tdiv_shift) )
				appr_mult = uint32( ( (uint64(tnr.data[tnr.used-1])<<32) + uint64(tnr.data[tnr.used-2]) ) / div );
			else
				appr_mult = uint32( uint64(tnr.data[tnr.used-1]) / div );

			if( appr_mult == 0 )
				appr_mult = 1;

			// tnr -= tdiv * 2^(32*tdiv_shift) * appr_mult;
			tnr.sub_uint32_mult_shifted( appr_mult, tdiv, tdiv_shift );

			if( ans ) // *ans += appr_mult * 2^(32*tdiv_shift)
				ans->add_uint32_shifted( appr_mult, tdiv_shift );
		}

		if( ans )
		{
			while( ans->used != 0 )
			{
				if( ans->data[ans->used-1] == 0 )
					ans->used--;
				else
					break;
			}
		}

		if( ans )
			ans->negative = negative ^ divider.negative;

		if( rest )
		{
			rest->swap( tnr );
			rest->negative = negative;
		}
	}
}

void Integer::add(const Integer &val) throw(error_obj)
{
	if( negative == val.negative )
		add_data( val );
	else
	{
		if( val.negative )
		{
			if( compare_abs_value( val ) >= 0 )
			{
				sub_data( val );
				negative = false;
			}
			else
			{
				sub_data_reverse( val );
				negative = true;
			}
		}
		else
		{
			if( compare_abs_value( val ) > 0 )
			{
				sub_data( val );
				negative = true;
			}
			else
			{
				sub_data_reverse( val );
				negative = false;
			}
		}
	}
}

void Integer::sub(const Integer &val) throw(error_obj)
{
	if( negative != val.negative )
		add_data( val );
	else
	{
		if( !negative )
		{
			if( compare_abs_value( val ) >= 0 )
			{
				sub_data( val );
				negative = false;
			}
			else
			{
				sub_data_reverse( val );
				negative = true;
			}
		}
		else
		{
			if( compare_abs_value( val ) > 0 )
			{
				sub_data( val );
				negative = true;
			}
			else
			{
				sub_data_reverse( val );
				negative = false;
			}
		}
	}
}

uint32 Integer::count_used_bits(void) const
{
	uint32 tmp, i;

	if( used == 0 )
		return 0;

	tmp = data[used-1];
	for( i=0; tmp; i++ )
		tmp >>= 1;

	return (used-1)*32 + i;
}

void Integer::set_bit( uint32 bit ) throw(error_obj)
{
	enlarge_to( bit/32 +1 );

	if( (bit/32 +1) > used )
	{
		for( uint32 i=used; i<(bit/32 +1); i++ )
			data[i] = 0;

		used = bit/32 +1;
	}

	data[bit/32] |= uint32(1) << (bit%32);
}

void Integer::clear_bit( uint32 bit ) throw(error_obj)
{
	if( (bit/32 +1) > used )
		return;

	data[bit/32] &= ~(uint32(1) << (bit%32));
	if( used == (bit/32 +1) )
	{
		for( ; used != 0  &&  data[used-1] == 0; used-- )
			;
	}
}

bool Integer::check_bit( uint32 bit ) const throw(error_obj)
{
	if( (bit/32 +1) > used )
		return false;

	if( data[bit/32] & (uint32(1) << (bit%32)) )
		return true;
	return false;
}

Integer iSqrt( const Integer &var ) throw(error_obj)
{
	Integer x, tmp, tmp2;
	bool root_found = false;
	int int_tmp;

	if( var.negative )
		Throw_Domain_error( "number must be positive in iSqrt" );

	if( var.used == 0 )
	{
		x.used = 0;
		return x;
	}

	x.enlarge_to( (var.used+1)/2 ); // makes room for the start guess
	{
		uint32 ans_bits = (var.count_used_bits()+1)/2;
		x.set_bit( ans_bits-1 ); // make the start guess
		if( ans_bits >= 3 )
		{
			x.set_bit( ans_bits-3 );
			if( !(var.count_used_bits() & 1) )
				x.set_bit( ans_bits-2 );
		}
	}

	while( !root_found )
	{
// nexton-raphson method for solving sqrt p = x
		var.divide( x, &tmp2, 0 );  // X[n+1] = ( X[n] + p/X[n] ) / 2
		x.add( tmp2 );
		x.div_by_2();

// the rest of the while-loop is just for checking when we are finished
		x.mul( x, &tmp );
		for( int o=0; o<2; o++ )
		{
			int_tmp = tmp.compare_abs_value( var );

			if( int_tmp < 0 )
			{
				tmp2 = x;
				tmp2.mul_by_2();
				tmp.add( tmp2 );
				tmp++;

				int_tmp = tmp.compare_abs_value( var );

				if( int_tmp < 0  &&  o == 0 )
					x++;
				else if( int_tmp == 0 )
				{
					x++;
					root_found = true;
					break;
				}
				else
				{
					root_found = true;
					break;
				} 
			}
			else if( int_tmp == 0 )
			{
				root_found = true;
				break;
			}
			else
				break;
		}
	}
	return x;
}

void Integer::power( Integer nr, Integer *dest ) const throw(error_obj)
{
	Integer tmp;
	uint32 power;

	if( dest == 0 )
		return;

	if( used == 0 )
	{
		if( nr.used == 0 )
			Throw_Domain_error( "can't raise 0 to 0" );
		else
		{
			dest->used = 0;
			return;
		}
	}

	if( nr.used > 1 )
	{
		if( used == 1  &&  data[0] == 1 )
		{
			dest->enlarge_to( 1 );
			dest->used = 1;
			dest->negative = false;
			dest->data[0] = 1;
			return;
		}
		else
			Throw_Overflow();
	}

	if( nr.used == 0 )
	{
		dest->enlarge_to( 1 );
		dest->used = 1;
		dest->negative = false;
		dest->data[0] = 1;
		return;
	}

	power = nr.data[0];
	nr = *this;

	if( power & 1 )
		*dest = *this;
	else
		*dest = 1;

	while( power >>= 1 )
	{
		nr.mul( nr, &tmp );
		nr = tmp;

		if( power & 1 )
		{
			dest->mul( nr, &tmp );
			*dest = tmp;
		}
	}
}

void Integer::mul_data( const Integer &factor, Integer *dest ) const throw(error_obj)
{
	uint32 i, i2, *ptr, *ptr_factor, *ptr_dest;
	uint64 tmp, overflow=0;

	if( dest == 0 )
		return;

	dest->enlarge_to( used + factor.used + 1 );
	dest->used = dest->size;

	for( i=0; i<dest->used; i++ )
		dest->data[i] = 0;

	ptr = data;
	for( i=0; i<used; i++, ptr++ )
	{
		ptr_factor = factor.data;
		ptr_dest = &(dest->data[i]);
		for( i2=0; i2<factor.used; i2++, ptr_factor++, ptr_dest++ )
		{
			tmp =  uint64(*ptr_dest) + uint64(*ptr) * uint64(*ptr_factor) + overflow;

			*ptr_dest = uint32( tmp );

			overflow = tmp >> 32;
		}
		if( overflow )
		{
			for( ; ; i2++ )
			{
				tmp = uint64(dest->data[i+i2]) + overflow;
				dest->data[i+i2] = uint32( tmp );

				overflow = tmp >> 32;
				if( overflow == 0 )
					break;
			}
		}
	}

	for( i=dest->used-1; i != uint32_max; i-- )
	{
		if( dest->data[i] != 0 )
			break;
	}
	dest->used = i+1;
}

void Integer::mul( const Integer &factor, Integer *dest ) const throw(error_obj)
{
	if( dest == 0 )
		return;

	//if( used < 4  ||  factor.used < 4 )
		mul_data( factor, dest );
	//else
	if( false )
	{  // here is a Divide & Conquer multiply implementation, but for some reason it crashes
		Integer ah, al, bh, bl, X;
		uint32 base;

		base = ( (used>=factor.used) ? used : factor.used ) /2;

		ah.enlarge_to( base * 3 );
		al.enlarge_to( base * 3 );
		bh.enlarge_to( base * 3 );
		bl.enlarge_to( base * 3 );
		X.enlarge_to( base * 3 );

		extract( &ah, base, used-1 );
		extract( &al, 0, base-1 );
		factor.extract( &bh, base, factor.used-1 );
		factor.extract( &bl, 0, base-1 );

		ah.mul_data( bh, &X );     // X = ah*bh
		al.mul_data( bl, dest );   // dest = al*bl
		ah.add_data( al );         // ah += al 
		bh.add_data( bl );         // bh += bl
		al.mul_data( bh, &ah );    // al = ah*bh = (ah+al)*(bh+bl)
		al.sub_data( X );          // al -= X
		al.sub_data( *dest );      // al -= dest

		for( ; base != 0; base-- )
		{
			X.mul_by_2_pow_32();
			X.mul_by_2_pow_32();
			al.mul_by_2_pow_32();
		}
		dest->add_data( al ); //, base );
		dest->add_data( X ); //, base*2 );
	}
	dest->negative = negative ^ factor.negative;
}

void Integer::faculty(void) throw(error_obj)
{
	if( used >= 2 )
		Throw_Overflow();

	if( negative )
		Throw_Domain_error( "can't calculate faculty of a negative number" );

	if( used == 0 )
	{
		enlarge_to( 1 );
		data[0] = 1;
		used = 1;
		return;
	}

	Integer tmp, tmp2;

	tmp = *this;

	for( tmp.data[0]--; tmp.data[0] > uint32(1); tmp.data[0]-- )
	{
		this->mul( tmp, &tmp2 );
		*this = tmp2;
	}
}

void Integer::perm( Integer n, Integer k ) throw(error_obj)
{
	if( n.negative  ||  k.negative )
		Throw_Domain_error( "can't calculate perm(n,k) if n or k is negative" );

	if( n.compare_abs_value( k ) < 0 )
		Throw_Domain_error( "can't calculate perm(n,k) when k > n" );

	Integer tmp;
	tmp = n;
	tmp.sub( k );
	k = tmp;

	*this = 1;

	for( ; n>k; n-- )
	{
		this->mul( n, &tmp );
		*this = tmp;
	}
}

void Integer::comb( const Integer &n, Integer k ) throw(error_obj)
{
	if( n.negative  ||  k.negative )
		Throw_Domain_error( "can't calculate comb(n,k) if n or k is negative" );

	if( n.compare_abs_value( k ) < 0 )
		Throw_Domain_error( "can't calculate comb(n,k) when k > n" );

	Integer tmp;

	tmp = n;
	tmp.sub( k );
	if( tmp < k )
		k = tmp;

	tmp.perm( n, k );
	k.faculty();

	tmp.divide( k, this, 0 );
}

void Integer::extract( Integer *dest, uint32 start, uint32 stop ) const throw(error_obj)
{
	if( dest == 0 )
		return;

	if( used == 0 )
	{
		*dest = 0;
		return;
	}
	if( stop >= used )
		stop = used -1;

	if( start > stop )
		return;

	dest->enlarge_to( stop-start+1 );
	dest->used = stop-start+1;

	for( uint32 i=0; start <= stop; i++, start++ )
		dest->data[i] = data[start];

	if( dest->data[dest->used-1] == 0 )
	{
		for( ; dest->used != 0  &&  dest->data[dest->used-1] == 0 ; dest->used-- )
			;
	}
}
